#  Drakkar-Software OctoBot-Services
#  Copyright (c) Drakkar-Software, All rights reserved.
#
#  This library is free software; you can redistribute it and/or
#  modify it under the terms of the GNU Lesser General Public
#  License as published by the Free Software Foundation; either
#  version 3.0 of the License, or (at your option) any later version.
#
#  This library is distributed in the hope that it will be useful,
#  but WITHOUT ANY WARRANTY; without even the implied warranty of
#  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
#  Lesser General Public License for more details.
#
#  You should have received a copy of the GNU Lesser General Public
#  License along with this library.

from octobot_services.notification import notification
from octobot_services.notification.notification import (
    Notification,
)
from octobot_services.notification import formated_notifications

from octobot_services.notification.formated_notifications import (
    OrderCreationNotification,
    OrderEndNotification,
)

__all__ = [
    "OrderCreationNotification",
    "OrderEndNotification",
    "Notification",
]
